
% Author: Sujoy Biswas
% Affiliation: University of California, Santa Cruz
% History: April, 2016

%% Compile mex file
clc; clear all; close all; 
fprintf('\n Compiling LSK mexfile...');
mex getlsk.c

%% Load Cropped Images from OSU Thermal Data
% Data source: http://vcipl-okstate.org/pbvs/bench/Data/01/download.html
clear all, close all;
fprintf('\n Preparing to run...\n');
% cropped image dimension
height = 31; width = 25;
% LSK parameters
conf.Wsize = 3; % LSK window size for aggregating gradients
conf.nbors = 3; % # LSK coeffeiencts (i.e., neighborhood of central pixel)
conf.h = 2.0; % smoothing parameter for LSK
conf.alpha = 0.4; % LSK sensitivity parameter
% compute mean from single image (1) or from full dataset (0)
compute_mean_optn = 1; 

% add the path/to/directory containing OSU Thermal image sequences
% Sequences are from 01 to 10
prfx = ('../osu_thermal_dir/');

allPeds = get_all_cropped_pedestrians(height, width, prfx);

%% LSK channels in one image
wsz = conf.nbors;
half = (wsz - 1)/2;
[nr, nc] = meshgrid(1+half : wsz: height, 1+half : wsz: width);
nr = nr'; nc = nc';
neighbor = [nr(:) nc(:)]; % neighbor locations

%% Compute mean LSK over all datapoints
if ~compute_mean_optn
    Wall = [];
    for nlist = 1:length(allPeds)
        img = allPeds{nlist};
        LSK = computeLSK(img, conf);
        LSKreshaped = reshape(LSK,[size(LSK,1)*size(LSK,2),size(LSK,3)*size(LSK,4)])';
        Wall = [Wall LSKreshaped];
    end
    premean = mean(Wall,2);
end
%% Compute the Principal LSK channels
% Compute the HOG channels
% Visualize LSK vs HOG
count = 0;
no_eigvec = 3;
for nlist = 1:length(allPeds)
    img = allPeds{nlist};
    LSK = computeLSK(img, conf);
    LSKreshaped = reshape(LSK,[size(LSK,1)*size(LSK,2),size(LSK,3)*size(LSK,4)])';
    N = size(LSKreshaped,2);
    
    % Compute mean LSK from single datapoint
    if compute_mean_optn, premean = mean(LSKreshaped, 2); end
    
    meanMat = repmat(premean,[1 N]);

    [v,d] = eig((LSKreshaped-meanMat)*(LSKreshaped-meanMat)');
    [sortEval, l] = sort(diag(d),'descend');
    v = v(:,l);

    s1 = v(1, :) < 0; sa = s1*(-1);
    sa = repmat(sa, [size(v, 1) 1]);
    s2 = ~s1; sb = repmat(s2, [size(v, 1) 1]);
    v = v.*sb + v.*sa;

    projectedL = (LSKreshaped-meanMat)'*v(:, 1:no_eigvec);
    F_T = reshape(projectedL, [size(LSK,1) size(LSK,2) no_eigvec]);
    
    viewL = zeros(height, width, 1);
    for eachpatch = 1 : size(neighbor, 1)
        reshapeL = reshape(LSK(neighbor(eachpatch, 1), neighbor(eachpatch, 2), :), [wsz wsz]);
        viewL(neighbor(eachpatch, 1) - half:neighbor(eachpatch, 1) + half, ...
            neighbor(eachpatch, 2) - half:neighbor(eachpatch, 2) + half) = reshapeL;
    end

    figure(3), subplot(length(allPeds), no_eigvec+1, count*(no_eigvec+1)+1), imagesc(imresize(img, 3)), colormap gray, axis image, axis off;
    for nn = 1 : no_eigvec
        figure(3), subplot(length(allPeds), no_eigvec+1, count*(no_eigvec+1)+1+nn), imagesc(imresize(F_T(:, :, nn), 3)), colormap gray, axis image, axis off;
    end
    
    hog = vl_hog(single(img), 4, 'verbose', 'variant', 'dalaltriggs');
    imhog = vl_hog('render', hog, 'verbose', 'variant', 'dalaltriggs') ;
    
    figure(5), subplot(length(allPeds), 3, count*3+1), imagesc(imresize(img, 3)), colormap gray, axis image, axis off;
    figure(5), subplot(length(allPeds), 3, count*3+2), imagesc(imresize(imhog, 3)), colormap gray, axis image, axis off;
    figure(5), subplot(length(allPeds), 3, count*3+3), imagesc(imresize(viewL, 3)), colormap gray, axis image, axis off;

    figure(7), subplot(length(allPeds), 3, count*3+1), imagesc(imresize(img, 3)), colormap(gca, 'gray'), axis image, axis off;
    figure(7), subplot(length(allPeds), 3, count*3+2), imagesc(imresize(imhog, 3)), colormap(gca, 'jet'), axis image, axis off;
    figure(7), subplot(length(allPeds), 3, count*3+3), imagesc(imresize(viewL, 3)), colormap(gca, 'jet'), axis image, axis off;
    
    count = count + 1;
end



